--     Island Explorer - An AIO island expeditions assistant
--     Copyright (C) 2019  Nivix
-- 
--     This program is free software: you can redistribute it and/or modify
--     it under the terms of the GNU General Public License as published by
--     the Free Software Foundation, either version 3 of the License, or
--     (at your option) any later version.
-- 
--     This program is distributed in the hope that it will be useful,
--     but WITHOUT ANY WARRANTY; without even the implied warranty of
--     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--     GNU General Public License for more details.
-- 
--     You should have received a copy of the GNU General Public License
--     along with this program.  If not, see <https://www.gnu.org/licenses/>.

local IE = IslandExplorer;
local islands = IE.islands;
local difficulty = 
        {
            [38] = "Normal",
            [39] = "Heroic",
            [40] = "Mythic",
            [45] = "PvP"
        };

function islands:updateFrame()
    islands.frame.islandName:SetText(C_Map.GetMapInfo(islands.currentZone).name .. " (" .. (GetDifficultyInfo(islands.instanceDifficulty)):sub(1,1) .. ")");
    islands.frame.bestTime:SetText("Best: " .. math.floor(islands.data[islands.currentZone][difficulty[islands.instanceDifficulty]].best / 60) .. "m" .. islands.data[islands.currentZone][difficulty[islands.instanceDifficulty]].best % 60 .. "s");
    islands.frame.averageTime:SetText("Count: " .. islands.data[islands.currentZone][difficulty[islands.instanceDifficulty]].count);
end

function islands:updateData()
    local time = IE.timer.minutes * 60 + IE.timer.seconds;
    if islands.state ~= "FINISHED" or time == 0 then
        return;
    end
    local toUpdate = { };
    toUpdate[1] = islands.data[islands.currentZone][difficulty[islands.instanceDifficulty]];
    toUpdate[2] = islands.data[947][difficulty[islands.instanceDifficulty]];
    for idx, islandData in pairs(toUpdate) do
        if islandData.best > time or islandData.count == 0 then
            islandData.best = time;
        end
        islandData.totalTime = islandData.totalTime + time;
        islandData.count = islandData.count + 1;
    end
end

function islands:setState()
	local buffInfo = {AuraUtil.FindAuraByName((GetSpellInfo(279737)), "player", "HARMFUL")};
	if buffInfo[1] ~= nil then
		islands.state = "WAITING";
        return;
    end
    islands.state = "IN PROGRESS";
end

function islands:onEnter()
    IE.frame:RegisterEvent("ISLAND_AZERITE_GAIN");
    IE.frame:RegisterEvent("ISLAND_COMPLETED");
    IE.frame:RegisterEvent("UNIT_AURA");
    IE.frame:RegisterEvent("LFG_COMPLETION_REWARD");
	local instanceInfo = {GetInstanceInfo()};
	islands.instanceDifficulty = instanceInfo[3];
    IE.scoreboard:reset();
    IE.timer:reset();
    IE.frame:Show();
    islands:updateFrame();
    islands:setState();
end

function islands:onLeave()
    IE.frame:UnregisterEvent("ISLAND_AZERITE_GAIN");
    IE.frame:UnregisterEvent("UNIT_AURA");
    IE.frame:UnregisterEvent("ISLAND_COMPLETED");
    IE.frame:UnregisterEvent("LFG_COMPLETION_REWARD");
    IE.timer:stop();
    islands.state = "NOT IN ISLAND";
end

function islands:onStart()
	IE.timer:start();
end

function islands:onFinish()
    IE.timer:stop();
    islands.state = "FINISHED";
    islands:updateData();
    islands:updateFrame();
    if IslandExplorerOptions["report"] and IE.timer.seconds + IE.timer.minutes > 0 then
        IE.scoreboard:chatReport();
    end
end

function islands:checkForStart()
    if islands.state == "WAITING" then
        islands.setState();
        if islands.state == "IN PROGRESS" then
            islands:onStart();
        end
    end
end

function islands:init()
    if IslandExplorerData == nil then
        IslandExplorerData = { };
    end
    for idx, island in pairs({"Havenswood", "The Rotting Mire",  "Verdant Wilds", "Jorundall", "Molten Cay", "Un'gol Ruins", "Whispering Reef", "The Dread Chain", "Skittering Hollow", "Crestfall", "Snowblossom Village", "All Islands"}) do
        if IslandExplorerData[island] == nil then
            IslandExplorerData[island] = { };
            for idx2, difficulty in pairs({"Normal", "Heroic", "Mythic", "PvP"}) do
                IslandExplorerData[island][difficulty] = { };
                for idx3, name in pairs({"best", "totalTime", "count"}) do
                   IslandExplorerData[island][difficulty][name] = 0;  
                end
            end
        end
    end
    islands.data =
            {
                [981] = IslandExplorerData["Un'gol Ruins"],
                [1032] = IslandExplorerData["Skittering Hollow"],
                [1033] = IslandExplorerData["The Rotting Mire"],
                [1034] = IslandExplorerData["Verdant Wilds"],
                [1035] = IslandExplorerData["Molten Cay"],
                [1036] = IslandExplorerData["The Dread Chain"],
                [1037] = IslandExplorerData["Whispering Reef"],
                [1336] = IslandExplorerData["Havenswood"],
                [1337] = IslandExplorerData["Jorundall"],
                [1501] = IslandExplorerData["Crestfall"],
                [1502] = IslandExplorerData["Snowblossom Village"],
                [947] = IslandExplorerData["All Islands"]
            };
    islands.frame = IslandExplorerFrame.islandInfo;
    islands.currentZone = 947;
    islands.instanceDifficulty = 40;
    islands.state = "NOT IN ISLAND";
    IE.islands:updateFrame();
end
