--     Island Explorer - An AIO island expeditions assistant
--     Copyright (C) 2019  Nivix
-- 
--     This program is free software: you can redistribute it and/or modify
--     it under the terms of the GNU General Public License as published by
--     the Free Software Foundation, either version 3 of the License, or
--     (at your option) any later version.
-- 
--     This program is distributed in the hope that it will be useful,
--     but WITHOUT ANY WARRANTY; without even the implied warranty of
--     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--     GNU General Public License for more details.
-- 
--     You should have received a copy of the GNU General Public License
--     along with this program.  If not, see <https://www.gnu.org/licenses/>.

local IE = IslandExplorer;
local scoreboard = IE.scoreboard;

local function initPlayer(unitID)
    scoreboard[unitID] = { };
    local entry = scoreboard[unitID];
    local name = UnitName(unitID)
    if name ~= nil then
        entry.player = name;
        local guid = UnitGUID(unitID);
        scoreboard[guid] = entry;
    else
        scoreboard[unitID].player = unitID;
    end
    entry.score = 0;
    entry.nameFrame = IslandExplorerFrame.scoreboard[unitID].name;
    entry.scoreFrame = IslandExplorerFrame.scoreboard[unitID].score;
    entry.nameFrame:SetText(entry.player);
    entry.scoreFrame:SetText(entry.score);
    local loc, class, index = UnitClass(unitID);
    local r, g, b = GetClassColor(class);
    entry.nameFrame:SetTextColor(r, g, b);
end

function scoreboard:update(playerGUID, amount)
    local scoreItem = scoreboard[playerGUID];
    if scoreItem ~= nil then
        scoreItem.score = scoreItem.score + amount;
        scoreItem.scoreFrame:SetText(scoreItem.score);
    else
        if UnitGUID("party1") == playerGUID then
            initPlayer("party1");
        elseif UnitGUID("party2") == playerGUID then
            initPlayer("party2");
        else
            print("Islex: Something went very wrong.");
        end
        scoreboard:update(playerGUID, amount);
    end
end

function scoreboard:scoreToString(unitID)
	return scoreboard[unitID].player .. ": " .. scoreboard[unitID].score;
end

function scoreboard:debugReport()
	print("Island Explorer - Report for " .. IE.frame.islandInfo.islandName:GetText());
    print(scoreboard:scoreToString("player"));
    print(scoreboard:scoreToString("party1"));
    print(scoreboard:scoreToString("party2"));
	print("Time: " .. IE.timer:toString());
    print(IE.frame.islandInfo.bestTime:GetText() .. ", " .. IE.frame.islandInfo.averageTime:GetText());
end

function scoreboard:chatReport()
    local channel = "INSTANCE_CHAT"
    if IE.islands.state == "NOT IN ISLAND" then
        if IsInGroup() then
            channel = "PARTY"
        else
            channel = "SAY";
        end
    end
    local totalScore = scoreboard["player"].score + scoreboard["party1"].score + scoreboard["party2"].score; 
    if IE.islands.state ~= "NOT IN ISLAND" or totalScore ~= 0 then
        SendChatMessage("Island Explorer - Report for " .. IE.frame.islandInfo.islandName:GetText(), channel);
        SendChatMessage(scoreboard:scoreToString("player"), channel);
        SendChatMessage(scoreboard:scoreToString("party1"), channel);
        SendChatMessage(scoreboard:scoreToString("party2"), channel);
        SendChatMessage("Time: " .. IE.timer.toString(), channel);
    end
end

function scoreboard:reset()
    for index, player in pairs({"player", "party1", "party2"}) do
        initPlayer(player);
    end
end

function scoreboard:init()
    IE.frame.reportButton:SetScript("OnClick", scoreboard.chatReport);
    scoreboard:reset();
end
